# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
import csv
import six
import datetime

from driverng import MessageV2, Worklist
from driverng.splitted import Splitted, String

logger = logging.getLogger("driverng.drivers.illumipro")


class IllumiproResultsBase(Splitted):

    SEPARATOR = ","


class IllumiproResult(object):
    FIELDS = [
        String("block"),
        String("assay"),
        String("day"),
        String("month"),
        String("year"),
        String("hour"),
        String("minute"),
        String("second"),
        String("assay_name"),
        String("assay_version"),
        String("assay_crc"),
        String("sofware_version"),
        String("sample_id1"),
        String("result1"),
        String("sample_id2"),
        String("result2"),
        String("sample_id3"),
        String("result3"),
        String("sample_id4"),
        String("result4"),
        String("sample_id5"),
        String("result5"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for i, field in enumerate(fields):
            setattr(self, field.name, line[i])


class IllumiproResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: IllumiproResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = six.StringIO(data)
        reader = csv.reader(buf, delimiter=str(","))
        for line in reader:
            if not len(line):
                continue
            if len(line) > 5:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "illumipro", "label": "ILLUMIPRO MERIDIAN"}

    def parse(self, data):
        obj = IllumiproResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            if m.block == "Block":
                continue
            date = None
            if m.day and m.month and m.year and m.hour and m.minute and m.second:
                result_date = "{}{}{}{}{}{}".format(
                    m.day, m.month, m.year, m.hour, m.minute, m.second
                )
                try:
                    date = datetime.datetime.strptime(result_date, "%d%m%y%H%M%S")
                except ValueError as e:
                    logger.info(
                        "Mauvais format pour la date et l'heure du résultat %s", e
                    )
            if m.sample_id1 != "EMPTY WELL":
                sample = worklist.get_or_create(sample_id=m.sample_id1, type="patient")
                sample.results.add(code=m.assay_name, value=m.result1, date=date)
            if m.sample_id2 != "EMPTY WELL":
                sample = worklist.get_or_create(sample_id=m.sample_id2, type="patient")
                sample.results.add(code=m.assay_name, value=m.result2, date=date)
            if m.sample_id3 != "EMPTY WELL":
                sample = worklist.get_or_create(sample_id=m.sample_id3, type="patient")
                sample.results.add(code=m.assay_name, value=m.result3, date=None)
            if m.sample_id4 != "EMPTY WELL":
                sample = worklist.get_or_create(sample_id=m.sample_id4, type="patient")
                sample.results.add(code=m.assay_name, value=m.result4, date=None)
            if m.sample_id5 != "EMPTY WELL":
                sample = worklist.get_or_create(sample_id=m.sample_id5, type="patient")
                sample.results.add(code=m.assay_name, value=m.result5, date=None)
        return worklist
